"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .createdocumentfromurlparams import CreateDocumentFromURLParams, CreateDocumentFromURLParamsMetadata, CreateDocumentFromURLParamsMetadataTypedDict, CreateDocumentFromURLParamsMode, CreateDocumentFromURLParamsTypedDict
from .createdocumentparams import CreateDocumentParams, CreateDocumentParamsMetadata, CreateDocumentParamsMetadataTypedDict, CreateDocumentParamsTypedDict, File, FileTypedDict, Mode
from .createdocumentrawparams import CreateDocumentRawParams, CreateDocumentRawParamsMetadata, CreateDocumentRawParamsMetadataTypedDict, CreateDocumentRawParamsTypedDict, Data, DataTypedDict, Two, TwoTypedDict
from .createinstructionparams import CreateInstructionParams, CreateInstructionParamsFilter, CreateInstructionParamsFilterTypedDict, CreateInstructionParamsScope, CreateInstructionParamsTypedDict
from .deletedocumentop import DeleteDocumentRequest, DeleteDocumentRequestTypedDict
from .document import Document, DocumentTypedDict, Metadata, MetadataTypedDict
from .documentdelete import DocumentDelete, DocumentDeleteTypedDict
from .documentfileupdate import DocumentFileUpdate, DocumentFileUpdateTypedDict
from .documentlist import DocumentList, DocumentListTypedDict
from .documentmetadataupdate import DocumentMetadataUpdate, DocumentMetadataUpdateMetadata, DocumentMetadataUpdateMetadataTypedDict, DocumentMetadataUpdateTypedDict
from .documentrawupdate import DocumentRawUpdate, DocumentRawUpdateTypedDict
from .documentsummary import DocumentSummary, DocumentSummaryTypedDict
from .entity import Entity, EntityTypedDict
from .entitylist import EntityList, EntityListTypedDict
from .errormessage import ErrorMessage, ErrorMessageData
from .getdocumentop import GetDocumentRequest, GetDocumentRequestTypedDict
from .getdocumentsummaryop import GetDocumentSummaryRequest, GetDocumentSummaryRequestTypedDict
from .httpvalidationerror import HTTPValidationError, HTTPValidationErrorData
from .instruction import Filter, FilterTypedDict, Instruction, InstructionTypedDict, Scope
from .listdocumentsop import ListDocumentsRequest, ListDocumentsRequestTypedDict, ListDocumentsResponse, ListDocumentsResponseTypedDict
from .listentitiesbydocumentop import ListEntitiesByDocumentRequest, ListEntitiesByDocumentRequestTypedDict, ListEntitiesByDocumentResponse, ListEntitiesByDocumentResponseTypedDict
from .listentitiesbyinstructionop import ListEntitiesByInstructionRequest, ListEntitiesByInstructionRequestTypedDict, ListEntitiesByInstructionResponse, ListEntitiesByInstructionResponseTypedDict
from .pagination import Pagination, PaginationTypedDict
from .patchdocumentmetadataop import PatchDocumentMetadataRequest, PatchDocumentMetadataRequestTypedDict
from .patchdocumentmetadataparams import PatchDocumentMetadataParams, PatchDocumentMetadataParamsMetadata, PatchDocumentMetadataParamsMetadataTypedDict, PatchDocumentMetadataParamsTypedDict
from .retrieval import Retrieval, RetrievalTypedDict
from .retrieveparams import RetrieveParams, RetrieveParamsTypedDict
from .scoredchunk import ScoredChunk, ScoredChunkTypedDict
from .sdkerror import SDKError
from .security import Security, SecurityTypedDict
from .updatedocumentfileop import UpdateDocumentFileRequest, UpdateDocumentFileRequestTypedDict
from .updatedocumentfileparams import UpdateDocumentFileParams, UpdateDocumentFileParamsFile, UpdateDocumentFileParamsFileTypedDict, UpdateDocumentFileParamsMode, UpdateDocumentFileParamsTypedDict
from .updatedocumentrawop import UpdateDocumentRawRequest, UpdateDocumentRawRequestTypedDict
from .updatedocumentrawparams import Data2, Data2TypedDict, UpdateDocumentRawParams, UpdateDocumentRawParamsData, UpdateDocumentRawParamsDataTypedDict, UpdateDocumentRawParamsTypedDict
from .updateinstructionop import UpdateInstructionRequest, UpdateInstructionRequestTypedDict
from .updateinstructionparams import UpdateInstructionParams, UpdateInstructionParamsTypedDict
from .validationerror import Loc, LocTypedDict, ValidationError, ValidationErrorTypedDict

__all__ = ["CreateDocumentFromURLParams", "CreateDocumentFromURLParamsMetadata", "CreateDocumentFromURLParamsMetadataTypedDict", "CreateDocumentFromURLParamsMode", "CreateDocumentFromURLParamsTypedDict", "CreateDocumentParams", "CreateDocumentParamsMetadata", "CreateDocumentParamsMetadataTypedDict", "CreateDocumentParamsTypedDict", "CreateDocumentRawParams", "CreateDocumentRawParamsMetadata", "CreateDocumentRawParamsMetadataTypedDict", "CreateDocumentRawParamsTypedDict", "CreateInstructionParams", "CreateInstructionParamsFilter", "CreateInstructionParamsFilterTypedDict", "CreateInstructionParamsScope", "CreateInstructionParamsTypedDict", "Data", "Data2", "Data2TypedDict", "DataTypedDict", "DeleteDocumentRequest", "DeleteDocumentRequestTypedDict", "Document", "DocumentDelete", "DocumentDeleteTypedDict", "DocumentFileUpdate", "DocumentFileUpdateTypedDict", "DocumentList", "DocumentListTypedDict", "DocumentMetadataUpdate", "DocumentMetadataUpdateMetadata", "DocumentMetadataUpdateMetadataTypedDict", "DocumentMetadataUpdateTypedDict", "DocumentRawUpdate", "DocumentRawUpdateTypedDict", "DocumentSummary", "DocumentSummaryTypedDict", "DocumentTypedDict", "Entity", "EntityList", "EntityListTypedDict", "EntityTypedDict", "ErrorMessage", "ErrorMessageData", "File", "FileTypedDict", "Filter", "FilterTypedDict", "GetDocumentRequest", "GetDocumentRequestTypedDict", "GetDocumentSummaryRequest", "GetDocumentSummaryRequestTypedDict", "HTTPValidationError", "HTTPValidationErrorData", "Instruction", "InstructionTypedDict", "ListDocumentsRequest", "ListDocumentsRequestTypedDict", "ListDocumentsResponse", "ListDocumentsResponseTypedDict", "ListEntitiesByDocumentRequest", "ListEntitiesByDocumentRequestTypedDict", "ListEntitiesByDocumentResponse", "ListEntitiesByDocumentResponseTypedDict", "ListEntitiesByInstructionRequest", "ListEntitiesByInstructionRequestTypedDict", "ListEntitiesByInstructionResponse", "ListEntitiesByInstructionResponseTypedDict", "Loc", "LocTypedDict", "Metadata", "MetadataTypedDict", "Mode", "Pagination", "PaginationTypedDict", "PatchDocumentMetadataParams", "PatchDocumentMetadataParamsMetadata", "PatchDocumentMetadataParamsMetadataTypedDict", "PatchDocumentMetadataParamsTypedDict", "PatchDocumentMetadataRequest", "PatchDocumentMetadataRequestTypedDict", "Retrieval", "RetrievalTypedDict", "RetrieveParams", "RetrieveParamsTypedDict", "SDKError", "Scope", "ScoredChunk", "ScoredChunkTypedDict", "Security", "SecurityTypedDict", "Two", "TwoTypedDict", "UpdateDocumentFileParams", "UpdateDocumentFileParamsFile", "UpdateDocumentFileParamsFileTypedDict", "UpdateDocumentFileParamsMode", "UpdateDocumentFileParamsTypedDict", "UpdateDocumentFileRequest", "UpdateDocumentFileRequestTypedDict", "UpdateDocumentRawParams", "UpdateDocumentRawParamsData", "UpdateDocumentRawParamsDataTypedDict", "UpdateDocumentRawParamsTypedDict", "UpdateDocumentRawRequest", "UpdateDocumentRawRequestTypedDict", "UpdateInstructionParams", "UpdateInstructionParamsTypedDict", "UpdateInstructionRequest", "UpdateInstructionRequestTypedDict", "ValidationError", "ValidationErrorTypedDict"]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .document import Document, DocumentTypedDict
from .pagination import Pagination, PaginationTypedDict
from ragie.types import BaseModel
from typing import List, TypedDict


class DocumentListTypedDict(TypedDict):
    pagination: PaginationTypedDict
    documents: List[DocumentTypedDict]
    

class DocumentList(BaseModel):
    pagination: Pagination
    documents: List[Document]
    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .entity import Entity, EntityTypedDict
from .pagination import Pagination, PaginationTypedDict
from ragie.types import BaseModel
from typing import List, TypedDict


class EntityListTypedDict(TypedDict):
    pagination: PaginationTypedDict
    entities: List[EntityTypedDict]
    

class EntityList(BaseModel):
    pagination: Pagination
    entities: List[Entity]
    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ragie.types import BaseModel
from typing import TypedDict


class UpdateInstructionParamsTypedDict(TypedDict):
    active: bool
    r"""Whether the instruction is active. Active instructions are applied to documents when they're created or when their file is updated."""
    

class UpdateInstructionParams(BaseModel):
    active: bool
    r"""Whether the instruction is active. Active instructions are applied to documents when they're created or when their file is updated."""
    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ragie import utils
from ragie.types import BaseModel

class ErrorMessageData(BaseModel):
    detail: str
    


class ErrorMessage(Exception):
    r"""Unauthorized"""
    data: ErrorMessageData

    def __init__(self, data: ErrorMessageData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, ErrorMessageData)




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from ragie import models, utils
from ragie._hooks import HookContext
from ragie.types import BaseModel, OptionalNullable, UNSET
from typing import Any, Optional, Union, cast

class Retrievals(BaseSDK):
    
    
    def retrieve(
        self, *,
        request: Union[models.RetrieveParams, models.RetrieveParamsTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.Retrieval]:
        r"""Retrieve

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms
        
        if server_url is not None:
            base_url = server_url
        
        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.RetrieveParams)
        request = cast(models.RetrieveParams, request)
        
        req = self.build_request(
            method="POST",
            path="/retrievals",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(request, False, False, "json", models.RetrieveParams),
            timeout_ms=timeout_ms,
        )
        
        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, [
                "429",
                "500",
                "502",
                "503",
                "504"
            ])                
        
        http_res = self.do_request(
            hook_ctx=HookContext(operation_id="Retrieve", oauth2_scopes=[], security_source=self.sdk_configuration.security),
            request=req,
            error_status_codes=["401","422","4XX","5XX"],
            retry_config=retry_config
        )
        
        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Optional[models.Retrieval])
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ErrorMessageData)
            raise models.ErrorMessage(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX","5XX"], "*"):
            raise models.SDKError("API error occurred", http_res.status_code, http_res.text, http_res)
        
        content_type = http_res.headers.get("Content-Type")
        raise models.SDKError(f"Unexpected response received (code: {http_res.status_code}, type: {content_type})", http_res.status_code, http_res.text, http_res)

    
    
    async def retrieve_async(
        self, *,
        request: Union[models.RetrieveParams, models.RetrieveParamsTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[models.Retrieval]:
        r"""Retrieve

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms
        
        if server_url is not None:
            base_url = server_url
        
        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.RetrieveParams)
        request = cast(models.RetrieveParams, request)
        
        req = self.build_request(
            method="POST",
            path="/retrievals",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(request, False, False, "json", models.RetrieveParams),
            timeout_ms=timeout_ms,
        )
        
        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, [
                "429",
                "500",
                "502",
                "503",
                "504"
            ])                
        
        http_res = await self.do_request_async(
            hook_ctx=HookContext(operation_id="Retrieve", oauth2_scopes=[], security_source=self.sdk_configuration.security),
            request=req,
            error_status_codes=["401","422","4XX","5XX"],
            retry_config=retry_config
        )
        
        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Optional[models.Retrieval])
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.ErrorMessageData)
            raise models.ErrorMessage(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX","5XX"], "*"):
            raise models.SDKError("API error occurred", http_res.status_code, http_res.text, http_res)
        
        content_type = http_res.headers.get("Content-Type")
        raise models.SDKError(f"Unexpected response received (code: {http_res.status_code}, type: {content_type})", http_res.status_code, http_res.text, http_res)

    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from pydantic import model_serializer
from ragie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from typing import Dict, List, TypedDict, Union
from typing_extensions import NotRequired


MetadataTypedDict = Union[str, int, bool, List[str]]


Metadata = Union[str, int, bool, List[str]]


class DocumentTypedDict(TypedDict):
    id: str
    created_at: datetime
    updated_at: datetime
    status: str
    name: str
    metadata: Dict[str, MetadataTypedDict]
    chunk_count: NotRequired[Nullable[int]]
    external_id: NotRequired[Nullable[str]]
    

class Document(BaseModel):
    id: str
    created_at: datetime
    updated_at: datetime
    status: str
    name: str
    metadata: Dict[str, Metadata]
    chunk_count: OptionalNullable[int] = UNSET
    external_id: OptionalNullable[str] = UNSET
    
    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["chunk_count", "external_id"]
        nullable_fields = ["chunk_count", "external_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (self.__pydantic_fields_set__.intersection({n}) or k in null_default_fields) # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
        



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .entitylist import EntityList, EntityListTypedDict
from pydantic import model_serializer
from ragie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from ragie.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing import Callable, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class ListEntitiesByInstructionRequestTypedDict(TypedDict):
    instruction_id: str
    r"""The ID of the instruction."""
    cursor: NotRequired[Nullable[str]]
    r"""An opaque cursor for pagination"""
    page_size: NotRequired[int]
    r"""The number of items per page (must be greater than 0 and less than or equal to 100)"""
    

class ListEntitiesByInstructionRequest(BaseModel):
    instruction_id: Annotated[str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))]
    r"""The ID of the instruction."""
    cursor: Annotated[OptionalNullable[str], FieldMetadata(query=QueryParamMetadata(style="form", explode=True))] = UNSET
    r"""An opaque cursor for pagination"""
    page_size: Annotated[Optional[int], FieldMetadata(query=QueryParamMetadata(style="form", explode=True))] = 10
    r"""The number of items per page (must be greater than 0 and less than or equal to 100)"""
    
    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["cursor", "page_size"]
        nullable_fields = ["cursor"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (self.__pydantic_fields_set__.intersection({n}) or k in null_default_fields) # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
        

class ListEntitiesByInstructionResponseTypedDict(TypedDict):
    result: EntityListTypedDict
    

class ListEntitiesByInstructionResponse(BaseModel):
    next: Callable[[], Optional[ListEntitiesByInstructionResponse]]
    
    result: EntityList
    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ragie.types import BaseModel
from typing import TypedDict


class DocumentDeleteTypedDict(TypedDict):
    status: str
    

class DocumentDelete(BaseModel):
    status: str
    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ragie.types import BaseModel
from ragie.utils import FieldMetadata, PathParamMetadata
from typing import TypedDict
from typing_extensions import Annotated


class DeleteDocumentRequestTypedDict(TypedDict):
    document_id: str
    r"""The id of the document."""
    

class DeleteDocumentRequest(BaseModel):
    document_id: Annotated[str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))]
    r"""The id of the document."""
    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from ragie.types import BaseModel
from typing import Any, Dict, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class RetrieveParamsTypedDict(TypedDict):
    query: str
    r"""The query to search with when retrieving document chunks."""
    top_k: NotRequired[int]
    r"""The maximum number of chunks to return. Defaults to 8."""
    filter_: NotRequired[Dict[str, Any]]
    r"""The metadata search filter on documents. Returns chunks only from documents which match the filter. The following filter operators are supported: $eq - Equal to (number, string, boolean), $ne - Not equal to (number, string, boolean), $gt - Greater than (number), $gte - Greater than or equal to (number), $lt - Less than (number), $lte - Less than or equal to (number), $in - In array (string or number), $nin - Not in array (string or number). The operators can be combined with AND and OR. Read [Metadata & Filters guide](https://docs.ragie.ai/docs/metadata-filters) for more details and examples."""
    rerank: NotRequired[bool]
    r"""Reranks the chunks for semantic relevancy post cosine similarity. Will be slower but returns a subset of highly relevant chunks. Best for reducing hallucinations and improving accuracy for LLM generation."""
    max_chunks_per_document: NotRequired[int]
    r"""Maximum number of chunks to retrieve per document. Use this to increase the number of documents the final chunks are retreived from. This feature is in beta and may change in the future."""
    

class RetrieveParams(BaseModel):
    query: str
    r"""The query to search with when retrieving document chunks."""
    top_k: Optional[int] = 8
    r"""The maximum number of chunks to return. Defaults to 8."""
    filter_: Annotated[Optional[Dict[str, Any]], pydantic.Field(alias="filter")] = None
    r"""The metadata search filter on documents. Returns chunks only from documents which match the filter. The following filter operators are supported: $eq - Equal to (number, string, boolean), $ne - Not equal to (number, string, boolean), $gt - Greater than (number), $gte - Greater than or equal to (number), $lt - Less than (number), $lte - Less than or equal to (number), $in - In array (string or number), $nin - Not in array (string or number). The operators can be combined with AND and OR. Read [Metadata & Filters guide](https://docs.ragie.ai/docs/metadata-filters) for more details and examples."""
    rerank: Optional[bool] = False
    r"""Reranks the chunks for semantic relevancy post cosine similarity. Will be slower but returns a subset of highly relevant chunks. Best for reducing hallucinations and improving accuracy for LLM generation."""
    max_chunks_per_document: Optional[int] = None
    r"""Maximum number of chunks to retrieve per document. Use this to increase the number of documents the final chunks are retreived from. This feature is in beta and may change in the future."""
    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, HttpClient
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, NoOpLogger
from .utils.retries import RetryConfig
import httpx
from ragie import models, utils
from ragie._hooks import SDKHooks
from ragie.documents import Documents
from ragie.entities import Entities
from ragie.retrievals import Retrievals
from ragie.types import OptionalNullable, UNSET
from typing import Any, Callable, Dict, Optional, Union

class Ragie(BaseSDK):
    documents: Documents
    retrievals: Retrievals
    entities: Entities
    def __init__(
        self,
        auth: Union[str, Callable[[], str]],
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param auth: The auth required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        if client is None:
            client = httpx.Client()

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        if async_client is None:
            async_client = httpx.AsyncClient()

        if debug_logger is None:
            debug_logger = NoOpLogger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."
        
        security: Any = None
        if callable(auth):
            security = lambda: models.Security(auth = auth()) # pylint: disable=unnecessary-lambda-assignment
        else:
            security = models.Security(auth = auth)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)
    

        BaseSDK.__init__(self, SDKConfiguration(
            client=client,
            async_client=async_client,
            security=security,
            server_url=server_url,
            server_idx=server_idx,
            retry_config=retry_config,
            timeout_ms=timeout_ms,
            debug_logger=debug_logger
        ))

        hooks = SDKHooks()

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(current_server_url, self.sdk_configuration.client)
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self._init_sdks()


    def _init_sdks(self):
        self.documents = Documents(self.sdk_configuration)
        self.retrievals = Retrievals(self.sdk_configuration)
        self.entities = Entities(self.sdk_configuration)
    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .updateinstructionparams import UpdateInstructionParams, UpdateInstructionParamsTypedDict
from ragie.types import BaseModel
from ragie.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import TypedDict
from typing_extensions import Annotated


class UpdateInstructionRequestTypedDict(TypedDict):
    instruction_id: str
    r"""The ID of the instruction."""
    update_instruction_params: UpdateInstructionParamsTypedDict
    

class UpdateInstructionRequest(BaseModel):
    instruction_id: Annotated[str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))]
    r"""The ID of the instruction."""
    update_instruction_params: Annotated[UpdateInstructionParams, FieldMetadata(request=RequestMetadata(media_type="application/json"))]
    



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from ragie.types import BaseModel
from typing import Dict, List, Optional, TypedDict, Union
from typing_extensions import NotRequired


CreateDocumentFromURLParamsMetadataTypedDict = Union[str, int, bool, List[str]]


CreateDocumentFromURLParamsMetadata = Union[str, int, bool, List[str]]


class CreateDocumentFromURLParamsMode(str, Enum):
    r"""Partition strategy for the document. Options are `'hi_res'` or `'fast'`. Only applicable for rich documents such as word documents and PDFs. When set to `'hi_res'`, images and tables will be extracted from the document. `'fast'` will only extract text. `'fast'` may be up to 20x faster than `'hi_res'`."""
    HI_RES = "hi_res"
    FAST = "fast"

class CreateDocumentFromURLParamsTypedDict(TypedDict):
    url: str
    r"""Url of the file to download. Must be publicly accessible and HTTP or HTTPS scheme"""
    metadata: NotRequired[Dict[str, CreateDocumentFromURLParamsMetadataTypedDict]]
    r"""Metadata for the document. Keys must be strings. Values may be strings, numbers, booleans, or lists of strings. Numbers may be integers or floating point and will be converted to 64 bit floating point. 1000 total values are allowed. Each item in an array counts towards the total. The following keys are reserved for internal use: `document_id`, `document_type`, `document_source`, `document_name`, `document_uploaded_at`."""
    mode: NotRequired[CreateDocumentFromURLParamsMode]
    r"""Partition strategy for the document. Options are `'hi_res'` or `'fast'`. Only applicable for rich documents such as word documents and PDFs. When set to `'hi_res'`, images and tables will be extracted from the document. `'fast'` will only extract text. `'fast'` may be up to 20x faster than `'hi_res'`."""
    name: NotRequired[str]
    external_id: NotRequired[str]
    

class CreateDocumentFromURLParams(BaseModel):
    url: str
    r"""Url of the file to download. Must be publicly accessible and HTTP or HTTPS scheme"""
    metadata: Optional[Dict[str, CreateDocumentFromURLParamsMetadata]] = None
    r"""Metadata for the document. Keys must be strings. Values may be strings, numbers, booleans, or lists of strings. Numbers may be integers or floating point and will be converted to 64 bit floating point. 1000 total values are allowed. Each item in an array counts towards the total. The following keys are reserved for internal use: `document_id`, `document_type`, `document_source`, `document_name`, `document_uploaded_at`."""
    mode: Optional[CreateDocumentFromURLParamsMode] = CreateDocumentFromURLParamsMode.FAST
    r"""Partition strategy for the document. Options are `'hi_res'` or `'fast'`. Only applicable for rich documents such as word documents and PDFs. When set to `'hi_res'`, images and tables will be extracted from the document. `'fast'` will only extract text. `'fast'` may be up to 20x faster than `'hi_res'`."""
    name: Optional[str] = None
    external_id: Optional[str] = None
    
